@@ -11,7 +11,7 @@ from scipy.spatial import distance
 from simtk.openmm import unit
 
 from .ommtk import MetadynamicSimulation
-from .utils import calc_coverage, moving_average
+from .utils import calc_coverage, moving_average,get_minimum_path_trajectory
 
 
 class SegmentMappingSim(MetadynamicSimulation):
@@ -110,8 +110,6 @@ class SegmentMappingSim(MetadynamicSimulation):
         # if self.CV_list[0][0] not in ['Distance', 'Weighted Distances']:
         #     raise ValueError('Segment Map Sim without distance or weighted distances not yet supported')
 
-        if self.scan_bias < self.prod_bias:
-            print('WARNING: scan bias should be high (40-50) and prod bias should be low (5-10)')
 
         if self.fes_interval != self.traj_interval:
             print('WARNING: traj interval and fes interval are different, CV indices will not match trajectory frames')
@@ -135,7 +133,6 @@ class SegmentMappingSim(MetadynamicSimulation):
             self.dG_traces = []
             self.coverage_traces =[]
             self.CV_traces = []
-            chosen_CVs_all_replicas = []
             for i in range(self.num_replicas):
 
                 if self.retain_scan_bias==False:
@@ -146,6 +143,7 @@ class SegmentMappingSim(MetadynamicSimulation):
                         os.remove(os.path.join(self.cwd, match.string))
 
                 if self.scan_CV_list is not None:
+                    self.production_CV_list = self.CV_list
                     self.CV_list = self.scan_CV_list[i]
                     self._remove_custom_forces()
                     self._build_custom_forces()
@@ -203,47 +201,38 @@ class SegmentMappingSim(MetadynamicSimulation):
                 with open(os.path.join(self.cwd, 'coverage_trace.pickle'), 'rb') as handle:
                     self.coverage_traces.append(pickle.load(handle))
 
-                print('distances from scan: {}'.format(distances))
+                print('CV1 from scan: {}'.format(distances))
 
                 self.simulation.reporters.clear()
 
-                # find traj frames where the CV is evenly spaced along num_seg segments in distance CV but
-                # but optionally constrain CV2 values to a uniform distribution within a specified range
-                distance_targets = np.linspace(np.min(distances), self.unbound_distance, self.num_segs)
-                if self.CV2_target_range:
-                    #make sure min and max aren't outside the CV range
-                    min = np.max([self.CV_list[1][3],CVs[0][1]-self.CV2_target_range])
-                    max = np.min([CVs[0][1]+self.CV2_target_range,self.CV_list[1][4]])
-                    CV2_targets = np.random.uniform(min,max,self.num_segs)
-                    print('CV2_targets',CV2_targets)
+                #turns out the large volume guassians are pretty good at getting the positions of barriers, but not the
+                #relative depth of wells. So we take the minimum path (saddle_points) as the segment coords
 
-                segment_frames = []
-                chosen_CVs = []
-                for k,target in enumerate(distance_targets):
-                    if self.CV2_target_range:
-                        idx = np.argmin(distance.cdist([(target, CV2_targets[k])], CVs))
-                    else:
-                        idx = (np.abs(distances - target)).argmin()
-                    #if idx is already in the list, try 100 times to choose another with similar value
-                    counter=0
-                    while idx in segment_frames and counter<100:
-                        counter+=1
-                        target+=random.randint(-100,100)* .01
-                        if self.CV2_target_range:
-                            idx = np.argmin(distance.cdist([(target, CV2_targets[k])], CVs))
-                        else:
-                            idx = (np.abs(distances - target)).argmin()
-                    segment_frames.append(idx)
-                    chosen_CVs.append(distances[idx])
-                    print('choosing CV values {} {}'.format(idx,CVs[idx]))
+                with open(os.path.join(self.cwd, 'CV_values.pickle'), 'rb') as handle:
+                    CV_trace = pickle.load(handle)
 
-                #load trajectory and slice for chosen frames
+                fes = self.metaD.getFreeEnergy().value_in_unit(unit.kilocalorie_per_mole)
+
+                # load trajectory, fes, and CV_values
                 traj = mdtraj.load_hdf5(os.path.join(self.cwd,self.traj_out))
-                self.segment_coords.append(traj.xyz[segment_frames])
-                chosen_CVs_all_replicas.append(chosen_CVs)
 
-                #now we have the coordinates we use for seeding, get the seg steps and delete the bias file and previous
-                #objects
+                #TODO come up with hueristic to pick more reliable energy cutoff and order estimate
+                saddle_traj, segment_frames = get_minimum_path_trajectory(CV_trace=CV_trace,
+                                                             fes=fes,
+                                                             order=15,
+                                                             CV_list=self.CV_list,
+                                                             energy_cutoff=-5,
+                                                             frames_per_minima=2,
+                                                             traj=traj)
+
+                if len(saddle_traj)==0:
+                    raise ValueError('did not find any wells for this scan!')
+                print('found {} MFEP saddle points for use in segment mapping'.format(len(saddle_traj)))
+
+                # slice traj for chosen frames
+                self.segment_coords.append(saddle_traj.xyz)
+
+                #now we have the coordinates we use for seeding, delete the bias file and previous objects
                 pattern = re.compile('bias(.*)\.npy')
                 matches = [pattern.match(filename) for filename in os.listdir(self.cwd) if pattern.match(filename)!=None]
                 for match in matches:
@@ -296,6 +285,10 @@ class SegmentMappingSim(MetadynamicSimulation):
             self.segment_coords = self.input_segment_coords
             self.scan_times = self.input_scan_times
 
+        #if we used a scan CV we have to reset the production CV
+        if self.scan_CV_list is not None:
+            self.CV_list = self.production_CV_list
+
         # remove and rebuild the metaD wrapper force with the new metaD params
         self._remove_custom_forces()
         self.bias_factor = self.prod_bias
@@ -375,6 +368,10 @@ class SegmentMappingSimHREMD(SegmentMappingSim):
             self.segment_coords = self.input_segment_coords
             self.scan_times = self.input_scan_times
 
+        #if we used a scan CV we have to reset the production CV
+        if self.scan_CV_list is not None:
+            self.CV_list = self.production_CV_list
+
         # remove and rebuild the metaD wrapper force with the new metaD params
         self._remove_custom_forces()
         self.bias_factor = self.prod_bias
@@ -410,11 +407,13 @@ class SegmentMappingSimHREMD(SegmentMappingSim):
         attempted = 0
         dG_std = 100
         dG_displacement = 100
+        counter = 0
         continue_simulating=True
 
         while continue_simulating:
 
             #run the sim
+            counter+=1
             self.metaD.step(self.simulation, steps_per_swap_attempt)
             sim_time += self.swap_freq
             #get current coords and energy
@@ -424,18 +423,19 @@ class SegmentMappingSimHREMD(SegmentMappingSim):
 
             #load in new coordinates and get the new energy
             chosen_replica = np.random.randint(self.num_replicas)
-            chosen_frame_index =  np.random.randint(self.num_segs)
-            print('chose replica',chosen_replica, ' frame ',chosen_frame_index)
+            chosen_frame_index =  np.random.randint(len(segment_coords[chosen_replica]))
             proposed_frame = segment_coords[chosen_replica][chosen_frame_index]
             self.simulation.context.setPositions(proposed_frame * unit.nanometers)
             proposed_energy = self.simulation.context.getState(getEnergy=True).getPotentialEnergy()
             log_p_accept = proposed_energy-current_energy
-            current_cv1_distance = self.metaD.getCollectiveVariables(self.simulation)[0]
-            print('current cv1 distance', current_cv1_distance,'logp accept',log_p_accept)
+            current_cv_values = self.metaD.getCollectiveVariables(self.simulation)
+            if counter%10==0:
+                print('chose replica',chosen_replica, ' frame ',chosen_frame_index)
+                print('current cv1 distance', current_cv_values)
             #compare deltaU of swap and normalize high energy probability of acceptance by 2.5 (energy of room temp in kJ/mol)
             if log_p_accept<=0 * unit.kilojoules_per_mole \
                     or random.random() < np.exp(-(log_p_accept/2.494341741366065).value_in_unit(unit.kilojoule_per_mole)) \
-                    or current_cv1_distance>self.unbound_distance:
+                    or current_cv_values[0]>self.unbound_distance:
                 accepted +=1
                 attempted +=1
             else:
