@@ -1,6 +1,6 @@
 
 import numpy as np
-from scipy import signal
+from scipy import signal, spatial
 from simtk.openmm import unit
 
 
@@ -11,6 +11,9 @@ water_residues = ['HOH', 'TIP3', 'TIP3P', 'SPCE', 'SPC', 'TIP4PEW', 'WAT', 'OH2'
 
 lipid_residues = ['AR', 'CHL', 'DHA', 'LA', 'MY', 'OL', 'PA', 'PC', 'PE', 'PGR', 'PH-', 'PS', 'SA', 'SM', 'ST']
 
+nucleic_residues = ['DA5','DC5','DT3','DG','DG5','DT','RG5','DA3','THY','RA','RU5','DC3','A','T','RA3','G','RG3',
+                    'URA','DT5','RC5','RA5','RU3','DA','DG3','RC','GUA','DC','RC3','RG','RU','C','CYT','U','ADE']
+
 lig_residues = ['LIG', 'UNL']
 
 def select_atoms(parmed_structure, keyword_selection=None, ligand_resname=None, resid_selection=None):
@@ -94,9 +97,9 @@ def select_atoms(parmed_structure, keyword_selection=None, ligand_resname=None,
 
         for keyword in selection_keywords:
 
-            if keyword not in ['ligand', 'protein', 'lipids', 'water', 'noh', 'not']:
+            if keyword not in ['ligand', 'protein', 'lipids', 'water', 'nucleic', 'noh', 'not']:
                 raise ValueError("""Keyword {} could not be parsed. 
-                Options are noh; not; protein; ligand; lipids; water.""".format(keyword))
+                Options are noh; not; protein; ligand; lipids; nucleic; water.""".format(keyword))
 
             if keyword == 'protein':
                 protein_list = [i.idx for i in parmed_structure.atoms if i.residue.name in aa_residues]
@@ -113,12 +116,16 @@ def select_atoms(parmed_structure, keyword_selection=None, ligand_resname=None,
                 for i in ligand_list:
                     final_list.append(i)
 
-
             if keyword == 'lipid':
                 lipid_list=[i.idx for i in parmed_structure.atoms if i.residue.name in lipid_residues]
                 for i in lipid_list:
                     final_list.append(i)
 
+            if keyword == 'nucleic':
+                nucleic_list=[i.idx for i in parmed_structure.atoms if i.residue.name in nucleic_residues]
+                for i in nucleic_list:
+                    final_list.append(i)
+
             if keyword == 'noh':
                 remove_list = []
                 for i in final_list:
@@ -528,4 +535,54 @@ def dG_to_Ki(dG):
     """
     RT = unit.MOLAR_GAS_CONSTANT_R * 300 * unit.kelvin
     RT = RT.value_in_unit(unit.kilocalorie_per_mole)
-    return np.e**(dG/RT) * unit.molar
\ No newline at end of file
+    return np.e**(dG/RT) * unit.molar
+
+
+def get_fes_minima(fes, order, energy_cutoff):
+    fes = np.array(fes)
+    minimax = signal.argrelextrema(fes, np.less, order=order)
+    minimay = signal.argrelextrema(fes, np.less, order=order, axis=1)
+
+    y_direction = set()
+    for i in range(len(minimay[0])):
+        y_direction.add((minimay[0][i], minimay[1][i]))
+    x_direction = set()
+    for i in range(len(minimax[0])):
+        x_direction.add((minimax[0][i], minimax[1][i]))
+
+    min_set = x_direction.union(y_direction)
+    min_set = list(min_set)
+    x_mins = [min_set[i][0] for i in range(len(min_set))]
+    y_mins = [min_set[i][1] for i in range(len(min_set))]
+
+    minima = list(zip(y_mins, x_mins))
+    final_list = []
+    for i in range(len(minima)):
+        if fes[x_mins[i], y_mins[i]] < energy_cutoff:
+            final_list.append((x_mins[i], y_mins[i]))
+
+    return np.array(final_list)
+
+def get_minimum_path_trajectory(fes,CV_trace,CV_list,traj,order,energy_cutoff,frames_per_minima):
+    minima = get_fes_minima(fes,order,energy_cutoff)
+    frame_list = []
+    final_list = []
+    for k,minimum in enumerate(minima):
+        CV_minima = (minimum[1] / (np.shape(fes)[1] / (CV_list[1][4] - CV_list[1][3])),
+                             minimum[0] / (np.shape(fes)[0] / (CV_list[0][4] - CV_list[0][3])))
+        #get the distances from FES minima to each point in the traj
+        distances = spatial.distance.cdist([CV_minima], CV_trace)
+        indices = np.array([i for i in range(len(distances[0]))])
+        cutoff = 10
+        while len(indices[distances[0] < cutoff]) > frames_per_minima:
+            cutoff -= .001
+        chosen_indices = indices[distances[0] < cutoff]
+        for i in chosen_indices:
+            frame_list.append((i,fes[minimum[0],minimum[1]]))
+        print('minima {}: frame {}: fes value {}'.format(k,chosen_indices,fes[minimum[0],minimum[1]]))
+        final_list.append(chosen_indices)
+
+    final_list = np.unique(np.concatenate(final_list))
+    print(final_list)
+    chosen_traj = traj[final_list]
+    return chosen_traj, frame_list
\ No newline at end of file
